/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: icpciReg.c,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Globals:
 */

#if (DEBUG_ITIFG & 128)
const char  *cr_reg[] = { "con   ",
			  "acqcon",
			  "pitch ",
			  "aoix  ",
			  "bmsfl ",
			  "bmsfh ",
			  "bmssl ",
			  "bmssh ",
			  "aqstrt",
			  "aqadr ",
			  "intadr",
			  "intcon",
			  "wzoom ",
			  "mdata ", 
			  "error ",
			  "error ", 
			  "prtcon",
			  "outprt",
			  "inport" 
			};
#endif

/*
 * Prototypes:
 */

u_short cr_read_mm (itifg_t*, u_char, u_short, u_short, u_short);
u_char cr_read_ms (itifg_t*, u_char, u_short, u_char);
int cr_write_mm (itifg_t*, u_char, u_short, u_short, u_short, u_short);
int cr_write_ms (itifg_t*, u_char, u_short, u_char, u_char);

/*
 * General Read Handler for IC-PCI Control Registers
 */

/* 16 bit register access with mask and range checking */

u_short
cr_read_mm (iti, offset, mask, min, max)
     
     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */

{
  u_short tmp_val = 0;
  
  if (!iti->regs_io)
    tmp_val = iti_readw ((u_short*)(iti->v_cr_base + offset));
  else
    tmp_val = iti_inw (iti->io_cr_base + offset);
  
  tmp_val &= mask;
  
#if(0)
  if (tmp_val & mask) iti->errno = -EITI_MASK;
  
  if (tmp_val < min || tmp_val > max) iti->errno = -EITI_RANGE;
#endif
  
  if (!iti->regs_io)
    ITI_PRINT7A ("cr-readw  %s, %#06X &(%#06X) >%#06X <%#06X\n",
		 cr_reg[offset / sizeof(u_short)], tmp_val, mask, min, max);
  else
    ITI_PRINT7A("cr-inw %s, %#06X &(%#06X) >%#06X <%#06X\n",
		cr_reg[offset / sizeof(u_short)], tmp_val, mask, min, max);
  
  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

u_char
cr_read_ms (iti, offset, mask, shift)
     
     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char shift;	/* number of bits to shift right */
     
{
  u_short tmp_val = 0;

  if (!iti->regs_io)
    tmp_val = iti_readw ((u_short*)(iti->v_cr_base + offset));
  else
    tmp_val = iti_inw (iti->io_cr_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

  if (!iti->regs_io)
    ITI_PRINT7A("cr-readw  %s, %#1X &(%#06X) >>%2d\n",
		cr_reg[offset / sizeof(u_short)], tmp_val, mask, shift);
  else
    ITI_PRINT7A("cr-inw %s, %#1X &(%#06X) >>%2d\n",
		cr_reg[offset / sizeof(u_short)], tmp_val, mask, shift);

  return ((u_char)tmp_val);
}
  
/*
 * General Write Handler for Interface Control Registers
 */

/* 16 bit register access with mask and range checking */

int
cr_write_mm (iti, offset, mask, min, max, value)

     itifg_t * iti;	/* number of IC-PCI board attached to the computer */
     u_char offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */
     u_short value;	/* value to set to */

{
  u_short tmp_val = 0;

  if (value & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }
    
  if (value < min || value > max) {
    iti->errno = -EITI_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

  if (!iti->regs_io)
    ITI_PRINT7A("cr-writew %s, %#06X &(%#06X) >%#06X <%#06X\n",
		cr_reg[offset / sizeof(u_short)], tmp_val, mask, min, max);
  else
    ITI_PRINT7A("cr-outw %s, %#06X &(%#06X) >%#06X <%#06X\n",
		cr_reg[offset / sizeof(u_short)], tmp_val, mask, min, max);

  if (!iti->regs_io)
    iti_writew (tmp_val, (u_short*)(iti->v_cr_base + offset));
  else
    iti_outw (tmp_val, iti->io_cr_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

int
cr_write_ms (iti, offset, mask, shift, value)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_char offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char  shift;	/* number of bits to shift right */
     u_char  value;	/* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }

  /* this have to be atomar */
  /* iti_splhigh (iti); */

  if (!iti->regs_io)
    read_val = iti_readw ((u_short*)(iti->v_cr_base + offset));
  else
    read_val = iti_inw (iti->io_cr_base + offset);
  
  read_val &= ~mask;
  write_val = read_val | tmp_val;

  if (!iti->regs_io)
    ITI_PRINT7A("cr-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
		cr_reg[offset / sizeof(u_short)],
		read_val, write_val, tmp_val, mask, shift);
  else
    ITI_PRINT7A(" cr-outw %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
		cr_reg[offset / sizeof(u_short)],
		read_val, write_val, tmp_val, mask, shift);

  if (!iti->regs_io)
    iti_writew (write_val, (u_short*)(iti->v_cr_base + offset));
  else
    iti_outw (write_val, iti->io_cr_base + offset);
  
  /* iti_spllow (iti); */ /* end atomar */
  
  return (OK);
}
  
/*
 * Special Read Handler for IC-PCI Control Registers
 */

u_char
cr_get_bmstren (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_BMSTREN_MASK,
		      CR_BMSTREN_SHIFT));
}

u_char
cr_get_bmilace (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_BMILACE_MASK,
		      CR_BMILACE_SHIFT));
}

u_char
cr_get_pixsz (itifg_t *iti) {

  u_char tmp_val;

  tmp_val = cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_PIXSZ_MASK,
		      CR_PIXSZ_SHIFT);

#if(0)
  if (IS_REV_A && tmp_val == 3)
    iti->errno = -EITI_RANGE;
#endif

  if (tmp_val == CR_PIXSZ_24_32)
    return 4;

  return (++tmp_val);
}

u_char
cr_get_memcfg (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_MEMCFG_MASK,
		      CR_MEMCFG_SHIFT));
}

u_char
cr_get_vbstat (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_VBSTAT_MASK,
		      CR_VBSTAT_SHIFT));
}

u_char
cr_get_fldstat (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_FLDSTAT_MASK,
		      CR_FLDSTAT_SHIFT));
}

u_char
cr_get_smstat (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_SMSTAT_MASK,
		      CR_SMSTAT_SHIFT));
}

u_char
cr_get_amp (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_CON_OFFSET,
		      CR_AMP_MASK,
		      CR_AMP_SHIFT));
}

u_char
cr_get_tmode (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_ACQCON_OFFSET,
		      CR_TMODE_MASK,
		      CR_TMODE_SHIFT));
}

u_char
cr_get_fcnt (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_ACQCON_OFFSET,
		      CR_FCNT_MASK,
		      CR_FCNT_SHIFT) + 1);
}

u_char
cr_get_fldsel (itifg_t *iti) {

  u_char tmp_val;

  tmp_val = cr_read_ms (iti,
			CR_ACQCON_OFFSET,
			CR_FLDSEL_MASK,
			CR_FLDSEL_SHIFT);

#if(0)
  if (tmp_val == 1) iti->errno = -EITI_RANGE;
#endif

  return (tmp_val);
}

u_char
cr_get_acqmd (itifg_t *iti) {

  u_char tmp_val;

  tmp_val = cr_read_ms (iti,
			CR_ACQCON_OFFSET,
			CR_ACQMD_MASK,
			CR_ACQMD_SHIFT);

#if(0)
  if (tmp_val == 1) iti->errno = -EITI_RANGE;
#endif

  return (tmp_val);
}

u_char
cr_get_newaq (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_ACQCON_OFFSET,
		      CR_NEWAQ_MASK,
		      CR_NEWAQ_SHIFT));
}

u_char
cr_get_gstat (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_ACQCON_OFFSET,
		      CR_GSTAT_MASK,
		      CR_GSTAT_SHIFT));
}

u_char
cr_get_fstrt (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_ACQCON_OFFSET,
		      CR_FSTRT_MASK,
		      CR_FSTRT_SHIFT));
}

u_short
cr_get_pitch (itifg_t *iti) {

  u_short tmp_val;

  tmp_val = cr_read_mm (iti,
			CR_PITCH_OFFSET,
			CR_PITCH_MASK,
			CR_PITCH_MIN,
			CR_PITCH_MAX);

  return (tmp_val << CR_PITCH_SHIFT);
}

u_short
cr_get_aoix (itifg_t *iti) {

  u_short tmp_val;

  tmp_val =  cr_read_mm (iti,
			 CR_AOIX_OFFSET,
			 CR_AOIX_MASK,
			 CR_AOIX_MIN,
			 CR_AOIX_MAX);

  return (tmp_val << CR_AOIX_SHIFT);
}

u_long
cr_get_bmsf (itifg_t *iti) {

  u_short tmp_low, tmp_high;

  tmp_low = cr_read_mm (iti,
			CR_BMSFL_OFFSET,
			CR_BMSFL_MASK,
			CR_BMSFL_MIN,
			CR_BMSFL_MAX);

  if (iti->big_mem) {
    tmp_high = cr_read_mm (iti,
			   CR_BMSFH_OFFSET,
			   (CR_BMSFH_MASK << 1) + 1,
			   CR_BMSFH_MIN,
			   (CR_BMSFH_MAX << 1) + 1);
  } else {
    tmp_high = cr_read_mm (iti,
			   CR_BMSFH_OFFSET,
			   CR_BMSFH_MASK,
			   CR_BMSFH_MIN,
			   CR_BMSFH_MAX);
  }

  tmp_low &= CR_BMSFL_MASK;

  if (iti->big_mem) {
    tmp_high &= (CR_BMSFH_MASK << 1) + 1;    
  } else {
    tmp_high &= CR_BMSFH_MASK;
  }

  return ((tmp_low << CR_BMSFL_SHIFT) | ((u_long)tmp_high << CR_BMSFH_SHIFT));
}

u_long
cr_get_bmss (itifg_t *iti) {

  u_short tmp_low, tmp_high;

  tmp_low = cr_read_mm (iti,
			CR_BMSSL_OFFSET,
			CR_BMSSL_MASK,
			CR_BMSSL_MIN,
			CR_BMSSL_MAX);

  if (iti->big_mem) {
    tmp_high = cr_read_mm (iti,
			   CR_BMSSH_OFFSET,
			   (CR_BMSSH_MASK << 1) + 1,
			   CR_BMSSH_MIN,
			   (CR_BMSSH_MAX << 1) + 1);
  } else {
    tmp_high = cr_read_mm (iti,
			   CR_BMSSH_OFFSET,
			   CR_BMSSH_MASK,
			   CR_BMSSH_MIN,
			   CR_BMSSH_MAX);
  }

  tmp_low &= CR_BMSSL_MASK;
  
  if (iti->big_mem) {
    tmp_high &= (CR_BMSSH_MASK << 1) + 1;    
  } else {
    tmp_high &= CR_BMSSH_MASK;
  }
  
  return ((tmp_low << CR_BMSSL_SHIFT) | ((u_long)tmp_high << CR_BMSSH_SHIFT));
}

u_long
cr_get_aqstart (itifg_t *iti) {

  u_long tmp_val;
  
  if (iti->big_mem) {
    tmp_val = cr_read_mm (iti,
			  CR_AQSTART_OFFSET,
			  (CR_AQSTART_MASK << 1) + 1,
			  CR_AQSTART_MIN,
			  (CR_AQSTART_MAX << 1) + 1);
  } else {
    tmp_val = cr_read_mm (iti,
			  CR_AQSTART_OFFSET,
			  CR_AQSTART_MASK,
			  CR_AQSTART_MIN,
			  CR_AQSTART_MAX);
  }

  return (tmp_val << CR_AQSTART_SHIFT);
}

u_long
cr_get_aqadr (itifg_t *iti) {

  u_long tmp_val;

  if (iti->big_mem) {
    tmp_val = cr_read_mm (iti,
			  CR_AQADR_OFFSET,
			  (CR_AQADR_MASK << 1) + 1,
			  CR_AQADR_MIN,
			  (CR_AQADR_MAX << 1) + 1);
  } else {
    tmp_val = cr_read_mm (iti,
			  CR_AQADR_OFFSET,
			  CR_AQADR_MASK,
			  CR_AQADR_MIN,
			  CR_AQADR_MAX);
  }

  return (tmp_val << CR_AQADR_SHIFT);
}

u_long
cr_get_intadr (itifg_t *iti) {

  u_long tmp_val;

  if (iti->big_mem) {
    tmp_val = cr_read_mm (iti,
			  CR_INTADR_OFFSET,
			  (CR_INTADR_MASK << 1) + 1,
			  CR_INTADR_MIN,
			  (CR_INTADR_MAX << 1) + 1);
  } else {
    tmp_val = cr_read_mm (iti,
			  CR_INTADR_OFFSET,
			  CR_INTADR_MASK,
			  CR_INTADR_MIN,
			  CR_INTADR_MAX);
  }
  return (tmp_val << CR_INTADR_SHIFT);
}

u_char
cr_get_vbien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_VBIEN_MASK,
		      CR_VBIEN_SHIFT));
}

u_char
cr_get_ovbien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_OVBIEN_MASK,
		      CR_OVBIEN_SHIFT));
}

u_char
cr_get_evbien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_EVBIEN_MASK,
		      CR_EVBIEN_SHIFT));
}

u_char
cr_get_amien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_AMIEN_MASK,
		      CR_AMIEN_SHIFT));
}

u_char
cr_get_trigien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_TRIGIEN_MASK,
		      CR_TRIGIEN_SHIFT));
}

u_char
cr_get_adrien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_ADRIEN_MASK,
		      CR_ADRIEN_SHIFT));
}

u_char
cr_get_aqien (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_AQIEN_MASK,
		      CR_AQIEN_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_vbint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_VBINT_MASK,
		      CR_VBINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_ovbint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_OVBINT_MASK,
		      CR_OVBINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_evbint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_EVBINT_MASK,
		      CR_EVBINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_amint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_AMINT_MASK,
		      CR_AMINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_trigint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_TRIGINT_MASK,
		      CR_TRIGINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_adrint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_ADRINT_MASK,
		      CR_ADRINT_SHIFT));
}

/* old, use xcr_test_irq */
u_char
cr_get_aqint (itifg_t *iti) {

  return (cr_read_ms (iti,
		      CR_INTCON_OFFSET,
		      CR_AQINT_MASK,
		      CR_AQINT_SHIFT));
}

/* REV-B Only! */
u_char
cr_get_hdec (itifg_t *iti)
{
  if (IS_REV_A)
    return CR_DEC_BY_1;
  
  return (cr_read_ms (iti,
		      CR_WZOOM_OFFSET,
		      CR_HDEC_MASK,
		      CR_HDEC_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_vdec (itifg_t *iti)
{
  if (IS_REV_A)
    return CR_DEC_BY_1;

  return (cr_read_ms (iti,
		      CR_WZOOM_OFFSET,
		      CR_VDEC_MASK,
		      CR_VDEC_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_clip (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
 
  return (cr_read_ms (iti,
		      CR_WZOOM_OFFSET,
		      CR_CLIP_MASK,
		      CR_CLIP_SHIFT));  
}

/* PCVision Only! */
u_char
cr_get_bmzoom (itifg_t *iti)
{
  if (IS_AMPV)
    ;
  else
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_WZOOM_OFFSET,
		      CR_BMZOOM_MASK,
		      CR_BMZOOM_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_outstrb (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_OUTSTRB_MASK,
		      CR_OUTSTRB_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_inregenb (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INREGENB_MASK,
		      CR_INREGENB_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_instrbpol (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INSTRBPOL_MASK,
		      CR_INSTRBPOL_SHIFT));  

}

/* REV-B Only! */
u_char
cr_get_inregclr (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INREGCLR_MASK,
		      CR_INREGCLR_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_inpien (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INPIEN_MASK,
		      CR_INPIEN_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_inpintpol (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INPINTPOL_MASK,
		      CR_INPINTPOL_SHIFT));  
}

/* REV-B Only! old, use xcr_test_irq */
u_char
cr_get_inpint (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INPINT_MASK,
		      CR_INPINT_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_inprint (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INPRINT_MASK,
		      CR_INPRINT_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_inprstrb (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_INPRSTRB_MASK,
		      CR_INPRSTRB_SHIFT));  
}

/* PCVision Only! */
u_char
cr_get_pad16 (itifg_t *iti)
{
  if (IS_AMPV)
    ;
  else
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_ms (iti,
		      CR_PORTCON_OFFSET,
		      CR_PAD16_MASK,
		      CR_PAD16_SHIFT));  
}

/* REV-B Only! */
u_char
cr_get_outport (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }

  return (cr_read_mm (iti,
		      CR_OUTPORT_OFFSET,
		      CR_OUTPORT_MASK,
		      CR_OUTPORT_MIN,
		      CR_OUTPORT_MAX));
}

/* REV-B Only! */
u_char
cr_get_inport (itifg_t *iti)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_read_mm (iti,
		      CR_INPORT_OFFSET,
		      CR_INPORT_MASK,
		      CR_INPORT_MIN,
		      CR_INPORT_MAX));
}

/*
 * Special Write Handler for IC-PCI Control Registers
 */

int
cr_set_bmstren (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_CON_OFFSET,
		       CR_BMSTREN_MASK,
		       CR_BMSTREN_SHIFT,
		       value));
}
  
int
cr_set_bmilace (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_CON_OFFSET,
		       CR_BMILACE_MASK,
		       CR_BMILACE_SHIFT,
		       value));
}
  
int
cr_set_pixsz (itifg_t *iti, u_char value) {

  if (IS_REV_A && value == CR_PIXSZ_24_24) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  return (cr_write_ms (iti,
		       CR_CON_OFFSET,
		       CR_PIXSZ_MASK,
		       CR_PIXSZ_SHIFT,
		       value));
}
  
int
cr_set_memcfg (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_CON_OFFSET,
		       CR_MEMCFG_MASK,
		       CR_MEMCFG_SHIFT,
		       value));
}

int
cr_set_tmode (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_ACQCON_OFFSET,
		       CR_TMODE_MASK,
		       CR_TMODE_SHIFT,
		       value));
}
  
int
cr_set_fcnt (itifg_t *iti, u_char value) {

  if (--value > (CR_FCNT_MASK >> CR_FCNT_SHIFT)) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  return (cr_write_ms (iti,
		       CR_ACQCON_OFFSET,
		       CR_FCNT_MASK,
		       CR_FCNT_SHIFT,
		       value));
}
  
int
cr_set_fldsel (itifg_t *iti, u_char value) {

  if (value == 1) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

    return (cr_write_ms (iti,
			 CR_ACQCON_OFFSET,
			 CR_FLDSEL_MASK,
			 CR_FLDSEL_SHIFT,
			 value));
	    }
  
int
cr_set_acqmd (itifg_t *iti, u_char value) {

  if (value == 1) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

    return (cr_write_ms (iti,
			 CR_ACQCON_OFFSET,
			 CR_ACQMD_MASK,
			 CR_ACQMD_SHIFT,
			 value));
}
  
int
cr_set_pitch (itifg_t *iti, u_short value) {

  u_short tmp_val;

  if (value & ((1 << CR_PITCH_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }
  
  tmp_val = value >> CR_PITCH_SHIFT;

  return (cr_write_mm (iti,
		       CR_PITCH_OFFSET,
		       CR_PITCH_MASK,
		       CR_PITCH_MIN,
		       CR_PITCH_MAX,
		       tmp_val));
}

int
cr_set_aoix (itifg_t *iti, u_short value) {

  u_short tmp_val;

  if (value & ((1 << CR_AOIX_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }

  tmp_val = value >> CR_AOIX_SHIFT;

  return (cr_write_mm (iti,
		       CR_AOIX_OFFSET,
		       CR_AOIX_MASK,
		       CR_AOIX_MIN,
		       CR_AOIX_MAX,
		       tmp_val));
}

int
cr_set_bmsf (itifg_t *iti, u_long value) {

  u_short tmp_low, tmp_high;

  if (value & ((1 << CR_BMSFL_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }

  tmp_low = (value >> CR_BMSFL_SHIFT) & CR_BMSFL_MASK;

  if (iti->big_mem) {
    tmp_high = (value >> CR_BMSFH_SHIFT) & ((CR_BMSFH_MASK << 1) + 1);
  } else {
    tmp_high = (value >> CR_BMSFH_SHIFT) & CR_BMSFH_MASK;
  }

  if (cr_write_mm (iti,
		   CR_BMSFL_OFFSET,
		   CR_BMSFL_MASK,
		   CR_BMSFL_MIN,
		   CR_BMSFL_MAX,
		   tmp_low) < 0) return (ERROR);

  if (iti->big_mem) {
    return (cr_write_mm (iti,
			 CR_BMSFH_OFFSET,
			 (CR_BMSFH_MASK << 1) + 1,
			 CR_BMSFH_MIN,
			 (CR_BMSFH_MAX << 1) + 1,
			 tmp_high));
  } else {
    return (cr_write_mm (iti,
			 CR_BMSFH_OFFSET,
			 CR_BMSFH_MASK,
			 CR_BMSFH_MIN,
			 CR_BMSFH_MAX,
			 tmp_high));
  }
}

int
cr_set_bmss (itifg_t *iti, u_long value) {

  u_short tmp_low, tmp_high;

  if (value & ((1 << CR_BMSSL_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }

  tmp_low = (value >> CR_BMSSL_SHIFT) & CR_BMSSL_MASK;

  if (iti->big_mem) {
    tmp_high = (value >> CR_BMSSH_SHIFT) & ((CR_BMSSH_MASK << 1) + 1);
  } else {
    tmp_high = (value >> CR_BMSSH_SHIFT) & CR_BMSSH_MASK;
  }

  if (cr_write_mm (iti,
		   CR_BMSSL_OFFSET,
		   CR_BMSSL_MASK,
		   CR_BMSSL_MIN,
		   CR_BMSSL_MAX,
		   tmp_low) < 0) return (ERROR);

  if (iti->big_mem) {
    return (cr_write_mm (iti,
			 CR_BMSSH_OFFSET,
			 (CR_BMSSH_MASK << 1) + 1,
			 CR_BMSSH_MIN,
			 (CR_BMSSH_MAX << 1) + 1,
			 tmp_high));
  } else {
    return (cr_write_mm (iti,
			 CR_BMSSH_OFFSET,
			 CR_BMSSH_MASK,
			 CR_BMSSH_MIN,
			 CR_BMSSH_MAX,
			 tmp_high));
  }
  return (OK);
}

int
cr_set_aqstart (itifg_t *iti, u_long value) {

  u_long tmp_val;

  if (value & ((1 << CR_AQSTART_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }

  tmp_val = value >> CR_AQSTART_SHIFT;

  if (iti->big_mem) {
    return (cr_write_mm (iti,
			 CR_AQSTART_OFFSET,
			 (CR_AQSTART_MASK << 1) + 1,
			 CR_AQSTART_MIN,
			 (CR_AQSTART_MAX << 1) + 1,
			 (u_short)tmp_val));
  } else {
    return (cr_write_mm (iti,
			 CR_AQSTART_OFFSET,
			 CR_AQSTART_MASK,
			 CR_AQSTART_MIN,
			 CR_AQSTART_MAX,
			 (u_short)tmp_val));
  }
  return (OK);
}

int
cr_set_intadr (itifg_t *iti, u_long value) {

  u_long tmp_val;

  if (value & ((1 << CR_INTADR_SHIFT) - 1))
    {
      iti->errno = -EITI_MASK;
      return (ERROR);
    }

  tmp_val = value >> CR_INTADR_SHIFT;

  if (iti->big_mem) {
    return (cr_write_mm (iti,
			 CR_INTADR_OFFSET,
			 (CR_INTADR_MASK << 1) + 1,
			 CR_INTADR_MIN,
			 (CR_INTADR_MAX << 1) + 1,
			 (u_short)tmp_val));
  } else {
    return (cr_write_mm (iti,
			 CR_INTADR_OFFSET,
			 CR_INTADR_MASK,
			 CR_INTADR_MIN,
			 CR_INTADR_MAX,
			 (u_short)tmp_val));
  }
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_vbien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_VBIEN_MASK,
		       CR_VBIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_ovbien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_OVBIEN_MASK,
		       CR_OVBIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_evbien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_EVBIEN_MASK,
		       CR_EVBIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_amien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_AMIEN_MASK,
		       CR_AMIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_trigien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_TRIGIEN_MASK,
		       CR_TRIGIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_adrien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_ADRIEN_MASK,
		       CR_ADRIEN_SHIFT,
		       value));
}

/* old, use iti_[en|dis]able_irq */
int
cr_set_aqien (itifg_t *iti, u_char value) {

  return (cr_write_ms (iti,
		       CR_INTCON_OFFSET,
		       CR_AQIEN_MASK,
		       CR_AQIEN_SHIFT,
		       value));
}

/* REV-B Only! */   
int
cr_set_hdec (itifg_t *iti, u_char value)
{

  int bpp;

  if (IS_REV_A)
    {
      if (value == CR_DEC_BY_1)
	return OK;
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  bpp = cr_get_pixsz (iti);
  
  switch (value)
    {
    case CR_DEC_BY_1:
      if (iti->width % (8 * bpp))
	{
	  iti->errno = -EITI_MASK;
	  return (ERROR);
	}
      break;
    case CR_DEC_BY_2:
      if (iti->width % (16 * bpp))
	{
	  iti->errno = -EITI_MASK;
	  return (ERROR);
	}
      break;
    case CR_DEC_BY_4:
      if (iti->width % (32 * bpp))
	{
	  iti->errno = -EITI_MASK;
	  return (ERROR);
	}
      break;
    case CR_DEC_BY_8:
      if (iti->width % (64 * bpp))
	{
	  iti->errno = -EITI_MASK;
	  return (ERROR);
	}
      if (bpp != 1)	/* this is only supported for 8bit images */
	{
	  iti->errno = -EITI_NOSUPPORT;
	  return (ERROR);
	}
      break;
    default:
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_WZOOM_OFFSET,
		       CR_HDEC_MASK,
		       CR_HDEC_SHIFT,
		       value));
}	

/* REV-B Only! */   
int
cr_set_vdec (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      if (value == CR_DEC_BY_1)
	return OK;
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  switch (value)
    {
    case CR_DEC_BY_1:
    case CR_DEC_BY_2:
    case CR_DEC_BY_4:
    case CR_DEC_BY_8:
      break;
    default:
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }

  return (cr_write_ms (iti,
		       CR_WZOOM_OFFSET,
		       CR_VDEC_MASK,
		       CR_VDEC_SHIFT,
		       value));
}	

/* REV-B Only! */   
int
cr_set_clip (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_WZOOM_OFFSET,
		       CR_CLIP_MASK,
		       CR_CLIP_SHIFT,
		       value));
}	

/* PCVision Only! */
int
cr_set_bmzoom (itifg_t *iti, u_char value)
{
  if (IS_AMPV)
    ;
  else
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_WZOOM_OFFSET,
		       CR_BMZOOM_MASK,
		       CR_BMZOOM_SHIFT,
		       value));
}	

/* REV-B Only! */   
int 
cr_set_outstrb (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }

  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_OUTSTRB_MASK,
		       CR_OUTSTRB_SHIFT,
		       value));
 
}	

/* REV-B Only! */   
int 
cr_set_inregenb (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_INREGENB_MASK,
		       CR_INREGENB_SHIFT,
		       value));
}
	
/* REV-B Only! */   
int
cr_set_instrbpol (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_INSTRBPOL_MASK,
		       CR_INSTRBPOL_SHIFT,
		       value));

}	

/* REV-B Only! */   
int
cr_set_inregclr (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_INREGCLR_MASK,
		       CR_INREGCLR_SHIFT,
		       value));

}	

/* REV-B Only! */   
int
cr_set_inpien (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_INPIEN_MASK,
		       CR_INPIEN_SHIFT,
		       value));

}
	
/* REV-B Only! */   
int
cr_set_inpintpol (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_INPINTPOL_MASK,
		       CR_INPINTPOL_SHIFT,
		       value));
}	

/* PCVision Only! */
int
cr_set_pad16 (itifg_t *iti, u_char value)
{
  if (IS_AMPV)
    ;
  else
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  
  return (cr_write_ms (iti,
		       CR_PORTCON_OFFSET,
		       CR_PAD16_MASK,
		       CR_PAD16_SHIFT,
		       value));
}	

/* REV-B Only! */   
int
cr_set_outport (itifg_t *iti, u_char value)
{
  if (IS_REV_A)
    {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }

  return (cr_write_mm (iti,
		       CR_OUTPORT_OFFSET,
		       CR_OUTPORT_MASK,
		       CR_OUTPORT_MIN,
		       CR_OUTPORT_MAX,
		       (u_short)value));
}	

/*
 * Special Initialization Handler for IC-PCI Bitfield Control Registers
 */

int
cr_init_con (itifg_t *iti) {

  return (cr_write_mm (iti,
		       CR_CON_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_CON_INIT));
}
		
int
cr_init_acqcon (itifg_t *iti) {

  return (cr_write_mm (iti,
		       CR_ACQCON_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_ACQCON_INIT));
}

int
cr_init_intcon (itifg_t *iti) {

  return (cr_write_mm (iti,
		       CR_INTCON_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_INTCON_INIT));
}

/* REV-B Only! */   
int
cr_init_wzoom (itifg_t *iti)
{
  
  return (cr_write_mm (iti,
		       CR_WZOOM_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_WZOOM_INIT));
}

int
cr_init_mdata (itifg_t *iti) {
  
  return (cr_write_mm (iti,
		       CR_MDATA_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_MDATA_INIT));
}

/* REV-B Only! */   
int
cr_init_portcon (itifg_t *iti)
{
  
  return (cr_write_mm (iti,
		       CR_PORTCON_OFFSET,
		       CR_NO_MASK,
		       CR_NO_MIN,
		       CR_NO_MAX,
		       CR_PORTCON_INIT));
}

