/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: procfs.c,v 7.0 2000/10/25 09:07:58 cvs Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>

#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#ifdef linux
#  include <linux/limits.h>	/* PATH_MAX */
#  include <asm/page.h>		/* PAGE_SIZE */
#endif

#ifdef __NetBSD__
#  include <sys/syslimits.h>	/* PATH_MAX */
#  define PAGE_SIZE	getpagesize()
#endif

#include "itifgExt.h"
#include "iticonfig.h"

/* this have to be the same as in itifgAux.c */
const struct mod_names_t mod_names[] =
{
  { ICP_AMVS, "AM-VS" },
  { ICP_AMDG, "AM-DIG" },
  { ICP_AMPV, "PCVision" },
  { ITI_VMPCD, "PCDig" }
};

int
read_procfs (u_long *version,
	     char *date,
	     int *ret_boards,
	     int ret_modules[ITI_BOARDS_MAX])
{
  char file[PATH_MAX];
  int desc;

  int count, version_ok, date_ok, board, boards = 0, boards_ok,
    module, modules[ITI_BOARDS_MAX], modules_ok;

  char *buffer = alloca (PAGE_SIZE), *bufptr, *token, *endptr;

  if (ret_boards)
    *ret_boards = 0;
  
  if (ret_modules)
    for (board = 0; board < ITI_BOARDS_MAX; board++)
      ret_modules[board] = ICP_AMNO;
  
  if (version)
    *version = 0UL;

  if (date)
    *date = '\0';

  strcpy (file, PFS_PREFIX);
  strcat (file, PFS_STRING);

  if ((desc = open (file, O_RDONLY)) < 0)
    {
      if (errno == ENOENT)
	return -EITI_ENT;
      else
	return -EITI_SYS;
    }
  
  /* first parse */
  count = PAGE_SIZE;
  if ((count = read (desc, buffer, count)) < 1)
    return -EITI_SYS;

  version_ok = date_ok = boards_ok = FALSE;

  bufptr = buffer;
  token = strsep (&bufptr, " \n");
  if (token)
    do
      {
	if (version_ok && date_ok && boards_ok)
	  break;
	/* readout version and date */
	if (!version_ok && !strcmp (token, PFS_VERSION_IDENT))
	  {
	    int byte, bytes[4];
	    
	    for (byte = 3; byte >= 0; byte--)
	      {
		switch (byte)
		  {
		  case 3:
		  case 2:
		    token = strsep (&bufptr, ".");
		    break;
		  case 1:
		    token = strsep (&bufptr, "-");
		    break;
		  case 0:
		    token = strsep (&bufptr, " \n");
		    break;
		  }
		if (!token)
		  break;
		bytes[byte] = strtol (token, &endptr, 10);
		if (*endptr != '\0')
		  break;
	      }
	    if (byte < 0)
	      {
		if (version)
		  *version = bytes[3] << 24 | bytes[2] << 16 |
		    bytes[1] << 8 | bytes[0];
		version_ok = TRUE;
	      }
	    continue;
	  }
	if (!date_ok && !strcmp (token, PFS_DATE_IDENT))
	  {
	    if ((token = strsep (&bufptr, "\n")))
	      {
		if (date)
		strcpy (date, token);
		date_ok = TRUE;
	      }
	    continue;
	  }
	/* find the number of boards */
	if (!boards_ok && !strcmp (token, PFS_BOARDS_IDENT))
	  {
	    if ((token = strsep (&bufptr, " \n")))
	      {
		boards = strtol (token, &endptr, 10);
		if (*endptr == '\0')
		  boards_ok = TRUE;
	      }
	    continue;
	  }
      }
    while ((token = strsep (&bufptr, " \n")));
    
  if (version && !version_ok)
    return -EITI_FMT;
  if (date && !date_ok)
    return -EITI_FMT;
  if (ret_boards)
    {
      if (boards_ok)
	*ret_boards = boards;
      else
	return -EITI_FMT;
    }

  /* second parse */  
  if ((lseek (desc, 0UL, SEEK_SET)) != 0)
    return -EITI_SYS;
  if ((read (desc, buffer, count)) != count)
    return -EITI_SYS;
  
  bufptr = buffer;
  modules_ok = FALSE;
  token = strsep (&bufptr, " \n");
  if (token)
    do
      {
	/* find the correspondent modules */

	if (!strcmp (token, PFS_MODULES_IDENT))
	  {
	    for (board = 0; board < boards; board++)
	      {
		if ((token = strsep (&bufptr, " \n")))
		  for (module = 0;
		       module < sizeof(mod_names) / sizeof(mod_names[0]);
		       module++)
		    if (!strcmp (token, mod_names[module].name))
		      {
			modules[board] = mod_names[module].id;
			if (board == boards - 1)
			  modules_ok = TRUE;
		      }
	      }
	    break;
	  }
      }
    while ((token = strsep (&bufptr, " \n")));
  
  if (ret_modules)
    {
      if (modules_ok)
	memcpy (ret_modules, modules, sizeof(modules));
      else
	return -EITI_FMT;
    }
  
  if (close (desc) < 0)
    return -EITI_SYS;

  return OK;
}

